package jp.kwebs;

import java.net.URI;
import java.util.List;
import javax.enterprise.context.ApplicationScoped;
import javax.inject.Inject;
import javax.ws.rs.FormParam;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;
import org.eclipse.microprofile.rest.client.inject.RestClient;
import io.quarkus.qute.CheckedTemplate;
import io.quarkus.qute.TemplateInstance;

@ApplicationScoped
@Path("/view")
public class BookViewClient {

	@CheckedTemplate
	public static class Templates {
		public static native TemplateInstance messageTemplate(String message);
		public static native TemplateInstance bookTemplate(Book book);
		public static native TemplateInstance booksTemplate(List<Book> books);
		public static native TemplateInstance newBookTemplate(List<String>items);
		public static native TemplateInstance updateBookTemplate(Book book, List<String>items);
	}
	
	public static List<String> items = List.of("文芸","歴史","科学","教育","その他");
	
	@Inject
	@RestClient
	BookService bookService;
	
	
	@GET
	@Produces(MediaType.TEXT_HTML)
	@Path("/{id}")
	public TemplateInstance oneBookById(@PathParam("id") Long id) {
		
		Book book = bookService.getBookById(id);
		return Templates.bookTemplate(book);
	}
	
	@GET
	@Produces(MediaType.TEXT_HTML)
	public TemplateInstance allBooks() {
		List<Book> books = bookService.getAllBooks();
		return Templates.booksTemplate(books);
	}
	
	
	@GET
	@Produces(MediaType.TEXT_HTML)
	@Path("/newform")
	public TemplateInstance newBookForm() {
		return Templates.newBookTemplate(items);
	}
	
	@POST
	@Path("/new")
	public Response newBook(
				@FormParam("id") 	long id,
				@FormParam("title") String title,
				@FormParam("genre") String genre,
				@FormParam("author")String author,
				@FormParam("price") int price,
				@FormParam("stock") String stock
			) {
		
		Book book = new Book();
		book.id = id;
		book.title = title;
		book.genre = genre;
		book.author = author;
		book.price = price;
		book.stock = "on".equals(stock);
		
		try {		
			bookService.create(book);
			return Response.status(301)
                .location(URI.create("/view/newform"))
                .build();
		}
		catch(Exception e) {
			String message="★id= "+id+" は登録できません。";
			return Response.accepted(Templates.messageTemplate(message)).build();
		}
	}
	
	
	@GET
	@Produces(MediaType.TEXT_HTML)
	@Path("/updateForm/{id}")
	public TemplateInstance updateBook(@PathParam("id") long id) {
		Book book = bookService.getBookById(id);
		return Templates.updateBookTemplate(book, items);
	}

	@POST
	@Path("/update")
	public Response updateBook(
			@FormParam("id") 	long id,
			@FormParam("title") String title,
			@FormParam("genre") String genre,
			@FormParam("author")String author,
			@FormParam("price") int price,
			@FormParam("stock") String stock
		) {
	
		Book book = new Book();
		book.id = id;
		book.title = title;
		book.genre = genre;
		book.author = author;
		book.price = price;
		book.stock = "on".equals(stock);			
		
		bookService.update(book);
        return Response.status(301)
                .location(URI.create("/view"))
                .build();
	}

	@GET
	@Produces(MediaType.TEXT_HTML)
	@Path("/delete/{id}")
	public TemplateInstance deleteBook(@PathParam("id") long id) {

		bookService.delete(id);
		List<Book> books = bookService.getAllBooks();
		return Templates.booksTemplate(books);		
	}

}























